

New patches:

[Added CLOS / MOP infrastructure.
boris.smilga@gmail.com**20071223151000] {
hunk ./cl-json.asd 20
-                             (:file "decoder" :depends-on ("common"))
-                             (:file "encoder" :depends-on ("common"))
+                             (:file "objects" :depends-on ("package"))
+                             (:file "decoder" :depends-on ("common" "objects"))
+                             (:file "encoder" :depends-on ("common" "objects"))
addfile ./src/objects.lisp
hunk ./src/objects.lisp 1
-
+(in-package :json)
+
+(defvar *prototype-name* 'prototype)
+
+(defun find-class* (class-designator)
+  (if (typep class-designator
+             '(or standard-class structure-class built-in-class))
+      class-designator
+      (find-class class-designator)))
+
+(defun old-and-new-slots (bindings superclasses)
+  (let ((superclasses-slots
+         (mapcar #'slot-definition-name
+                 (reduce #'union (mapcar #'class-slots superclasses)
+                         :initial-value nil))))
+    (loop for binding in bindings
+      if (member (car binding) superclasses-slots) collect binding into old
+      else collect binding into new
+      finally (return (values old new)))))
+
+(defun make-and-populate-instance (class bindings)
+  (let ((object (make-instance class)))
+    (loop for (slot . value) in bindings
+      do (setf (slot-value object slot) value))
+    object))
+
+(defmethod make-object (bindings (class (eql nil)) &key (superclasses nil))
+  (let ((superclasses (mapcar #'find-class* superclasses)))
+    (multiple-value-bind (old-slot-bindings new-slot-bindings)
+        (old-and-new-slots bindings superclasses)
+      (if (and (null new-slot-bindings) (<= 0 (length superclasses) 1))
+          (setq class (or (first superclasses) 'standard-object))
+          (let ((anonymous-class
+                 (make-instance 'standard-class
+                   :direct-superclasses superclasses
+                   :direct-slots (mapcar
+                                  (lambda (binding) `(:name ,(car binding)))
+                                  new-slot-bindings))))
+            (dolist (superclass superclasses)
+              ;; The anonymous class should be removed from the
+              ;; superclasses' direct-subclasses lists right away because
+              ;; it is purely interim.  If it is not removed, it remains
+              ;; after its (single) instance is disused and so never GCs.
+              (remove-direct-subclass superclass anonymous-class))
+            (setq class anonymous-class)))
+      (make-and-populate-instance
+       class (nconc new-slot-bindings old-slot-bindings)))))
+
+(defmethod make-object (bindings class &key &allow-other-keys)
+  (let ((class (find-class* class)))
+    (make-and-populate-instance
+     class (old-and-new-slots bindings (list class)))))
+
+(defmethod make-object (bindings (class (eql (find-class 'cons)))
+                        &key &allow-other-keys)
+  (copy-seq bindings))
+
+(defmethod make-object (bindings (class (eql (find-class 'hash-table)))
+                        &key &allow-other-keys)
+  (let ((table (make-hash-table)))
+    (loop for (key . value) in bindings
+      do (setf (gethash key table) value))
+    table))
+
+(defmethod make-object (bindings (class symbol) &key &allow-other-keys)
+  (make-object bindings (find-class class)))
+
+(defclass prototype ()
+  ((lisp-class :initarg :lisp-class :reader lisp-class)
+   (lisp-superclasses :initarg :lisp-superclasses :reader lisp-superclasses)
+   (lisp-package :initarg :lisp-package :reader lisp-package))
+  (:default-initargs :lisp-class nil :lisp-superclasses nil
+                     :lisp-package '#:keyword))
+
+(defun max-package (symbols &key ((:initial-value package)
+                                  (find-package '#:common-lisp)))
+  (labels ((symbol-in-package-p (symbol)
+             (eq (find-symbol (symbol-name symbol) package) symbol))
+           (update-package-for-symbol (symbol)
+             (if (not (symbol-in-package-p symbol))
+                 (setq package (symbol-package symbol))))
+           (check-symbol (symbol)
+             (if (not (symbol-in-package-p symbol))
+                 (warn "Symbol ~S cannot be found in apparent package ~S."
+                       symbol package))))
+    (mapc #'update-package-for-symbol symbols)
+    (mapc #'check-symbol symbols)
+    package))
+
+(defun package-name* (package)
+  (let ((name (package-name package)))
+    (if (stringp name) (make-symbol name) name)))
+
+(defmethod make-object-prototype (object &optional slot-names)
+  (let* ((class (class-of object))
+         (class-name (if class (class-name class)))
+         (superclass-names
+          (if (not class-name)
+              (mapcar #'class-name (class-direct-superclasses class))))
+         (package
+          (max-package superclass-names
+            :initial-value
+              (max-package slot-names
+                :initial-value (if class-name
+                                   (symbol-package class-name)
+                                   (find-package '#:common-lisp))))))
+      (make-instance 'prototype
+        :lisp-class class-name
+        :lisp-superclasses superclass-names
+        :lisp-package (package-name* package))))
+
+(defmethod make-object-prototype ((object prototype) &optional slot-names)
+  (declare (ignore object slot-names))
+  nil)
+
+(defun maybe-add-prototype (object prototype)
+  (if prototype
+      (loop for slot in (class-slots (class-of object))
+        for slot-name = (slot-definition-name slot)
+        if (eql slot-name *prototype-name*)
+          return (setf (slot-value object slot-name) prototype)))
+  object)
+
+(defun make-object-slot-iterator (object)
+  (let ((slots
+         (loop for slot in (class-slots (class-of object))
+           for slot-name = (slot-definition-name slot)
+           if (slot-boundp object slot-name)
+             collect slot-name)))
+    (lambda ()
+      (loop until (endp slots)
+         for slot = (pop slots)
+         return (values t slot (slot-value object slot))))))
+
+(defmacro with-iterator-and-prototype (generator-fn object &body body)
+  (let ((add-proto (gensym)) (slot-names (gensym)))
+    `(let ((,add-proto (and *prototype-name* t))
+           (,slot-names nil))
+       (flet ((,generator-fn ()
+                (if (typep ,add-proto 'boolean)
+                    (multiple-value-bind (more name value) (,generator-fn)
+                      (cond
+                        (more
+                         (if (and ,add-proto (eq name *prototype-name*))
+                             (setq ,add-proto nil))
+                         (if (symbolp name)
+                             (push name ,slot-names))
+                         (values more name value))
+                        (,add-proto
+                         (let ((proto (make-object-prototype
+                                       ,object ,slot-names)))
+                           (if proto
+                               (values t *prototype-name*
+                                       (setq ,add-proto proto))))))))))
+         ,@body))))
hunk ./src/package.lisp 23
-    ))
+    )
+  (:import-from #+(or mcl openmcl) #:ccl
+                #+cmu #:clos-mop
+                #+sbcl #:sb-mop
+                #+(or clisp ecl lispworks) #:clos
+                #+allegro #:mop
+    #:slot-definition-name
+    #:class-slots
+    #:class-direct-superclasses
+    #:remove-direct-subclass
+    )
+   )
}

[Added decoding of objects to CLOS objects, hash tables or alists.
boris.smilga@gmail.com**20071223151334] {
hunk ./src/decoder.lisp 12
+(defparameter *json-object-prototype* nil)
hunk ./src/decoder.lisp 15
-                                                      (push (cons (json-intern key) value)
+                                                      (push (cons key value)
hunk ./src/decoder.lisp 17
-(defparameter *json-object-factory-return* #'(lambda (obj) (nreverse obj)))
+
+(defun json-factory-make-object (factory)
+  (flet ((intern-keys (alist package)
+           (let ((*json-symbols-package* package))
+             (loop for (key . value) in alist with ret
+               do (push (cons (json-intern key) value) ret)
+               finally (return ret)))))
+    (if (eq *json-object-prototype* t)
+        (make-object (intern-keys factory '#:json) (find-class 'prototype))
+        (multiple-value-bind (package class superclasses)
+            (if *json-object-prototype*
+                (values (lisp-package *json-object-prototype*)
+                        (lisp-class *json-object-prototype*)
+                        (lisp-superclasses *json-object-prototype*)))
+          (let* ((*json-symbols-package*
+                  (find-package
+                   (camel-case-to-lisp (string (or package '#:keyword)))))
+                 (bindings
+                  (intern-keys factory *json-symbols-package*)))
+            (maybe-add-prototype
+             (if class
+                 (make-object bindings
+                              (if (symbolp class)
+                                  class
+                                  (json-intern (string class))))
+                 (make-object bindings nil
+                   :superclasses (loop for super in superclasses
+                                   collect (json-intern (string super)))))
+             *json-object-prototype*))))))
+
+(defparameter *json-object-factory-return* #'json-factory-make-object)
+
hunk ./src/decoder.lisp 98
-  (let ((obj (funcall *json-object-factory*)))
+  (let ((obj (funcall *json-object-factory*))
+        (prototype *json-object-prototype*))
hunk ./src/decoder.lisp 102
-        (loop for skip-whitepace = (peek-char t stream)
-              for key = (read-json-string stream)
-              for separator = (peek-char t stream)
-              for skip-separator = (assert (char= #\: (read-char stream)))
-              for value = (decode-json stream)
+        (loop for key = (progn (peek-char t stream) (read-json-string stream))
+              with prototype-name =
+                (if *prototype-name*
+                    (funcall *symbol-to-string-fn* *prototype-name*)
+                    "")
+              for value =
+                (progn
+                  (if (and (not prototype) (string= key prototype-name))
+                      (setq prototype t))
+                  (peek-char t stream)
+                  (assert (char= #\: (read-char stream)))
+                  (let ((*json-object-prototype* prototype)
+                        (*json-array-type*
+                         ;; The list of superclasses should be read as list
+                         (if (eq prototype t) 'list *json-array-type*)))
+                    (decode-json stream)))
hunk ./src/decoder.lisp 119
-              for skip-terminator = (assert (member (read-char stream) '(#\, #\})))
-              do (setf obj (funcall *json-object-factory-add-key-value* obj key value))
+              do (assert (member (read-char stream) '(#\, #\})))
+              if (typep value 'prototype)
+                do (setf prototype value)
+              else
+                do (setf obj (funcall *json-object-factory-add-key-value* obj key value))
hunk ./src/decoder.lisp 125
-    (funcall *json-object-factory-return* obj)))
+    (let ((*json-object-prototype* prototype))
+      (funcall *json-object-factory-return* obj))))
hunk ./src/package.lisp 4
+    #:*prototype-name*
}

[Added decoding of arrays to vectors or lists.
boris.smilga@gmail.com**20071223151610] {
hunk ./src/decoder.lisp 96
+(defparameter *json-array-type* 'vector)
+
hunk ./src/decoder.lisp 134
-  (if (char= #\] (peek-char t stream))
-      (progn (read-char stream) nil)
-      (loop for first-in-element = (assert (not (member (peek-char t stream) '(#\, #\]))))
-            for element = (decode-json stream)
-            for terminator = (peek-char t stream)
-            for skip-terminator = (assert (member (read-char stream) '(#\, #\])))
-            collect element        
-            until (char= #\] terminator))))
+  (coerce
+   (if (char= #\] (peek-char t stream))
+       (progn (read-char stream) nil)
+       (loop for first-in-element = (assert (not (member (peek-char t stream) '(#\, #\]))))
+             for element = (decode-json stream)
+             for terminator = (peek-char t stream)
+             for skip-terminator = (assert (member (read-char stream) '(#\, #\])))
+             collect element
+             until (char= #\] terminator)))
+   *json-array-type*))
hunk ./src/package.lisp 9
+    #:*json-array-type*
}

[Added WITH-OLD-DECODER-SEMANTICS.
boris.smilga@gmail.com**20071223151705] {
hunk ./src/decoder.lisp 217
+(defmacro with-old-decoder-semantics (&body body)
+  `(let ((*json-object-prototype*
+          (make-instance 'prototype
+            :lisp-class 'cons :lisp-package "keyword"))
+         (*json-array-type* 'list))
+     ,@body))
+
hunk ./src/package.lisp 15
+    #:with-old-decoder-semantics
}

[Added encoding of CLOS objects, and prototypes for alists and hash tables.
boris.smilga@gmail.com**20071223151744] {
hunk ./src/encoder.lisp 65
-(defmethod encode-json((h hash-table) stream)
+(defmethod encode-json ((h hash-table) stream)
hunk ./src/encoder.lisp 67
-      (write-json-object generator stream)))
+    (with-iterator-and-prototype generator h
+      (write-json-object generator stream))))
+
+(defmethod encode-json ((o standard-object) stream)
+  (let ((gen-func (make-object-slot-iterator o)))
+    (macrolet ((generator () '(funcall gen-func)))
+      (with-iterator-and-prototype generator o
+        (write-json-object generator stream)))))
+
+(defmethod encode-json ((o structure-object) stream)
+  (let ((gen-func (make-object-slot-iterator o)))
+    (macrolet ((generator () '(funcall gen-func)))
+      (with-iterator-and-prototype generator o
+        (write-json-object generator stream)))))
hunk ./src/encoder.lisp 94
-    (write-json-object gen-fn stream)))
+    (with-iterator-and-prototype gen-fn alist
+      (write-json-object gen-fn stream))))
}

Context:

[json-rpc-error-object as in working draft fro json-rpc spec 1.1 
Henrik Hjelte <henrik@evahjelte.com>**20070531150713] 
[restart functions for json-rpc
Henrik Hjelte <henrik@evahjelte.com>**20070531134607] 
[documented parenscript dependency
Henrik Hjelte <henrik@evahjelte.com>**20070325211904] 
[TAG 0.3.1
Henrik Hjelte <henrik@evahjelte.com>**20070324142014] 
Patch bundle hash:
29f7329bf4dd51816c0514589a85876c00bed449

